-- reduced affine arithmetic (2 noise symbols + remainder; no rounding control)
-- x0 + x1e1 + x2e2 + x3x3 with x3>=0

local abs=math.abs
local aa={}
--aa.__index=aa

function aa.__tostring(t)
	return string.format("[%g;%g,%g,%g]",t[0],t[1],t[2],t[3])
end

local function aa_new(t)
	t[0],t[1],t[2],t[3],t[4]=t[1],t[2],t[3],t[4],nil
assert(t[3])
	return setmetatable(t,aa)
end

local function aa_norm(t)
	return abs(t[1])+abs(t[2])+abs(t[3])
end

function aa.interval(a,b,k)
	local t=aa_new{(b+a)/2,(b-a)/2,0,0}
	k = k or 1
	t[1],t[k]=t[k],t[1]
	return t
end

function aa.unpack(t)
	return t[0],t[1],t[2],t[3]
end

function aa.range(t)
	local w=aa_norm(t)
	return t[0]-w,t[0]+w
end

function aa.__unm(x)
	return aa_new{-x[0],-x[1],-x[2],x[3]}
end

function aa.__add(x,y)
	if type(x)=="number" then
		return aa_new{x+y[0],y[1],y[2],y[3]}
	elseif type(y)=="number" then
		return aa_new{x[0]+y,x[1],x[2],x[3]}
	else
		return aa_new{x[0]+y[0],x[1]+y[1],x[2]+y[2],x[3]+y[3]}
	end
end

function aa.__sub(x,y)
	if type(x)=="number" then
		return aa_new{x-y[0],-y[1],-y[2],y[3]}
	elseif type(y)=="number" then
		return aa_new{x[0]-y,x[1],x[2],x[3]}
	else
		return aa_new{x[0]-y[0],x[1]-y[1],x[2]-y[2],x[3]+y[3]}
	end
end

function aa.__mul(x,y)
	if type(x)=="number" then
		return aa_new{x*y[0],x*y[1],x*y[2],abs(x)*y[3]}
	elseif type(y)=="number" then
		return aa_new{x[0]*y,x[1]*y,x[2]*y,x[3]*abs(y)}
	else
		return aa_new{x[0]*y[0],x[0]*y[1]+x[1]*y[0],x[0]*y[2]+x[2]*y[0],
			abs(x[0])*y[3]+x[3]*abs(y[0])+aa_norm(x)*aa_norm(y)}
	end
end

local function aa_sqr(x)
	return aa_new{x[0]^2+x[1]^2/2+x[2]^2/2+x[3]^2,2*x[0]*x[1],2*x[0]*x[2],
		2*abs(x[0])*x[3]+2*abs(x[1])*x[3]+2*abs(x[2])*x[3]+aa_norm(x)^2/2}
end

function aa.__pow(x,y)
	if y==2 then
		--return x*x
		return aa_sqr(x)
	elseif y==3 then
		return (x^2)*x
	elseif y==4 then
		return (x^2)^2
	elseif y==5 then
		return (x^4)*x
	elseif y==6 then
		return (x^3)^2
	else
		error("bad y in aa.pow: "..tostring(y))
	end
end

aa.new=aa_new
aa.norm=aa_norm

return aa
