local aa=require"aa1"
local interval=aa.interval
local range=aa.range
local plotboxes=true
local plotgraph=true
local plotpoints=true
local plotzonotope=true

local function curve(t)
	local a=1
	local x=a*2*(t^2-3)
	local y=a*t*(t^2-3)
	return x,y
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function graph(xmin,xmax,ymin,ymax,tmin,tmax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dt=(tmax-tmin)/n
	local x1,y1=curve(tmin)
	local x2,y2
	for t=tmin,tmax,dt do
		x2,y2=curve(t)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=curve(tmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function zonotope(x,y)
	local x1=x[0]-x[1]
	local y1=y[0]-y[1]
	local x2=x[0]+x[1]
	local y2=y[0]+y[1]
	love.graphics.setColor(255,200,200)
	love.graphics.line(x1,y1,x2,y2)
	x[3],y[2],y[3]=0,0,y[2]
	local v
	if y[1]<0 then x[1],y[1]=-x[1],-y[1] end
	if x[1]>0 then
		v={x[2],0, x[1],y[1], 0,y[3], -x[2],0, -x[1],-y[1], 0,-y[3]}
	else
		v={x[2],0, 0,y[3], x[1],y[1], -x[2],0, 0,-y[3], -x[1],-y[1]}
	end
	local p={}
	p[1]=x[0]+v[1]+v[3]+v[5];	p[2]=y[0]+v[2]+v[4]+v[6]
	p[3]=x[0]+v[3]+v[5]+v[7];	p[4]=y[0]+v[4]+v[6]+v[8]
	p[5]=x[0]+v[5]+v[7]+v[9];	p[6]=y[0]+v[6]+v[8]+v[10]
	p[7]=x[0]+v[7]+v[9]+v[11];	p[8]=y[0]+v[8]+v[10]+v[12]
	p[9]=x[0]+v[9]+v[11]+v[1];	p[10]=y[0]+v[10]+v[12]+v[2]
	p[11]=x[0]+v[11]+v[1]+v[3];	p[12]=y[0]+v[12]+v[2]+v[4]
	love.graphics.setColor(255,0,0)
	love.graphics.polygon("line",p)
	love.graphics.setColor(255,200,200,100)
	love.graphics.polygon("fill",p)
end

local function box(tmin,tmax)
	local t=interval(tmin,tmax)
	local x,y=curve(t)
	if plotboxes then
		local xmin,xmax=range(x)
		local ymin,ymax=range(y)
		love.graphics.setColor(200,200,255)
		love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
		love.graphics.setColor(0,0,255)
		love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
	end
	if plotzonotope then
		zonotope(x,y)
	end
end

local function boxes(tmin,tmax,n)
	local dt=(tmax-tmin)/n
	for t=tmin,tmax-dt,dt do
		box(t,t+dt)
	end
	box(tmax-dt,tmax)
end

local function point(t)
	local x,y=curve(t)
	love.graphics.points(x,y)
end

local function points(tmin,tmax,n)
	love.graphics.setColor(100,100,100)
	local dt=(tmax-tmin)/n
	for t=tmin,tmax,dt do
		point(t)
	end
	point(tmax)
end

local function plot(xmin,xmax,ymin,ymax,tmin,tmax,n)
	coords(xmin,xmax,ymin,ymax)
	boxes(tmin,tmax,n)
	if plotgraph then
		graph(xmin,xmax,ymin,ymax,tmin,tmax,500)
	end
	if plotpoints then
		points(tmin,tmax,n)
	end
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("curve plotting with affine arithmetic")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
	love.graphics.setPointSize(8)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	elseif key == 'b' then
		plotboxes= not plotboxes
	elseif key == 'g' then
		plotgraph= not plotgraph
	elseif key == 'p' then
		plotpoints= not plotpoints
	elseif key == 'z' then
		plotzonotope= not plotzonotope
	end
end

function love.draw()
	plot(-7,5,-4,4,-2,2.2,2^N)
end
