local ia=require"ia"
local interval=ia.new
local plotgraph=true

local function f(t)
	local a=1
	local x=a*2*(t^2-3)
	local y=a*t*(t^2-3)
	return x,y
end

local function g(t)
	local x=t
	local y=t^2-1
	return x,y
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function graph(curve,xmin,xmax,tmin,tmax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dt=(tmax-tmin)/n
	local x1,y1=curve(tmin)
	local x2,y2
	for t=tmin,tmax,dt do
		x2,y2=curve(t)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=curve(tmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function box(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,255,100)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(0,0,255)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function accept(x1,y1,x2,y2)
	box(x1.min,x1.max,y1.min,y1.max)
	box(x2.min,x2.max,y2.min,y2.max)
	local xmin=math.max(x1.min,x2.min)
	local xmax=math.min(x1.max,x2.max)
	local ymin=math.max(y1.min,y2.min)
	local ymax=math.min(y1.max,y2.max)
	love.graphics.setColor(255,200,200)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(255,0,0)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function discard(xmin,xmax,ymin,ymax)
do return end
	love.graphics.setColor(200,200,255)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(0,0,255)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function disjoint(a,b)
	return a.max<b.min or b.max<a.min
end

local function explore(tmin,tmax,smin,smax,level)
	local t=interval(tmin,tmax)
	local s=interval(smin,smax)
	local x1,y1=f(t)
	local x2,y2=g(s)
	if disjoint(x1,x2) or disjoint(y1,y2) then
		discard(x1,y1,x2,y2)
	elseif level==0 then
		accept(x1,y1,x2,y2)
	else
		local tmid=(tmin+tmax)/2
		local smid=(smin+smax)/2
		explore(tmin,tmid,smin,smid,level-1)
		explore(tmin,tmid,smid,smax,level-1)
		explore(tmid,tmax,smin,smid,level-1)
		explore(tmid,tmax,smid,smax,level-1)
	end
end

local function plot(xmin,xmax,ymin,ymax,tmin,tmax,smin,smax,n)
	coords(xmin,xmax,ymin,ymax)
	explore(tmin,tmax,smin,smax,n)
	if plotgraph then
		graph(f,xmin,xmax,tmin,tmax,500)
		graph(g,xmin,xmax,smin,smax,500)
	end
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("curve intersection")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
	love.graphics.setPointSize(8)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	elseif key == 'g' then
		plotgraph= not plotgraph
	end
end

function love.draw()
	--plot(-7,5,-4,4,-2,2.2,-2,2,N)
	plot(-0.5,0.5,-2,2,-2,2.2,-2,2,N)
end
