local unpack = unpack or table.unpack
local a,b,c=1,-2,1

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function box(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,200)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(0,0,0)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function edge(x1,y1,x2,y2)
	local v1=a*x1+b*y1+c
	local v2=a*x2+b*y2+c
	if v2==0 or v1*v2>0 then
		return nil
	else
		local t=v1/(v1-v2)
		local x=(1-t)*x1+t*x2
		local y=(1-t)*y1+t*y2
		return x,y
	end
end

local function intersect(xmin,xmax,ymin,ymax)
	local x1,y1=edge(xmin,ymin,xmin,ymax)
	local x2,y2=edge(xmax,ymin,xmax,ymax)
	local x3,y3=edge(xmin,ymin,xmax,ymin)
	local x4,y4=edge(xmin,ymax,xmax,ymax)
	local p={}
	local n=0
	if x1 then n=n+1; p[n]=x1; n=n+1; p[n]=y1 end
	if x2 then n=n+1; p[n]=x2; n=n+1; p[n]=y2 end
	if x3 then n=n+1; p[n]=x3; n=n+1; p[n]=y3 end
	if x4 then n=n+1; p[n]=x4; n=n+1; p[n]=y4 end
	love.graphics.setColor(255,0,0)
	love.graphics.points(p)
end

local function plot(xmin,xmax,ymin,ymax)
	coords(xmin,xmax,ymin,ymax)
	xmin,xmax,ymin,ymax=0,1,0,1
	box(xmin,xmax,ymin,ymax)
	intersect(xmin,xmax,ymin,ymax)
	love.graphics.setColor(0,0,255)
	love.graphics.line((-b*(-5)-c)/a,-5, (-b*(5)-c)/a,5)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%.2f\t%.2f",a,c),10,10)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,640,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("floating-point numbers")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
	love.graphics.setPointSize(8)
end

function love.keypressed(key)
	local d=0.125
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
	elseif key == 'left' then
		a=a-d
	elseif key == 'right' then
		a=a+d
	elseif key == 'up' then
		c=c+d
	elseif key == 'down' then
		c=c-d
	end
end

function love.draw()
	plot(-1,2,-1,2)
end
