local ia=require"ia"
local interval=ia.new

-- https://en.wikipedia.org/wiki/Test_functions_for_optimization

local function f(x,y)
	--return (30+(2*x-3*y)^2*(18-32*x+12*x^2+48*y-36*x*y+27*y^2))
	--return (x-1.23)^2+(y-0.234)^2
	--return 0.26*(x^2+y^2)-0.48*x*y
	--return (x^2+y^2)
	--return (x-1.23456789)^2+(y-0.23456789)^2
	return ((x-1.23)^2+(y-0.234)^2)*(x^2+y^2)
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function accept(xmin,xmax,ymin,ymax)
	love.graphics.setColor(255,200,200)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(255,0,0)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function discard(xmin,xmax,ymin,ymax)
do return end
	love.graphics.setColor(200,200,255)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(0,0,255)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function putbox(L,xmin,xmax,ymin,ymax,level)
	table.insert(L,{xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax,level=level})
end

local function getbox(L)
	return table.remove(L,1)
end

local function openbox(b)
	return b.xmin,b.xmax,b.ymin,b.ymax,b.level
end

local function optimize(xmin,xmax,ymin,ymax,n)
	local accepted=0
	local inf=1/0
	local L={}
	local fmin=inf
	putbox(L,xmin,xmax,ymin,ymax,n)
	while true do
		local b=getbox(L)
		if b==nil then break end
		local xmin,xmax,ymin,ymax,level=openbox(b)
		local x=interval(xmin,xmax)
		local y=interval(ymin,ymax)
		local v=f(x,y)
		local vmin,vmax=v.min,v.max
		if vmin>fmin then
			discard(xmin,xmax,ymin,ymax)
		else
			if vmax<fmin then fmin=vmax end
			if level<0 then
				accept(xmin,xmax,ymin,ymax)
				accepted=accepted+1
			elseif level==0 then
				putbox(L,xmin,xmax,ymin,ymax,level-1)
			else
				local xmid=(xmin+xmax)/2
				local ymid=(ymin+ymax)/2
				putbox(L,xmin,xmid,ymin,ymid,level-1)
				putbox(L,xmin,xmid,ymid,ymax,level-1)
				putbox(L,xmid,xmax,ymin,ymid,level-1)
				putbox(L,xmid,xmax,ymid,ymax,level-1)
			end
		end
	end
	return fmin,accepted
end

local function result(fmin,n,accepted)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%d\t%.4f\t%d",n,fmin,accepted),10,10)
end

local function plot(xmin,xmax,ymin,ymax,n)
	coords(xmin,xmax,ymin,ymax)
	local fmin,accepted=optimize(xmin,xmax,ymin,ymax,n)
	result(fmin,n,accepted)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("global optimization with interval arithmetic - branch and bound 2d")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	end
end

function love.draw()
	local xmin,xmax=-1.125,2.5
	local ymin,ymax=-2.25,1.5
	plot(xmin,xmax,ymin,ymax,N)
end
