-- interval arithmetic

local ia={}

local function machine()
	local eps=1.0
	repeat
		eps=eps/2.0
	until (1.0+eps)==1.0
	eps=eps*2.0
	local a=1.0
	local min
	repeat
		min=a
		a=a/2.0
	until a==0
	return eps,min
end

local fp_eps,fp_min=machine()

local function nextf(x)
	if x>0 then
		return x*(1.0+fp_eps)
	elseif x<0 then
		return x*(1.0-fp_eps)
	else
		return 0	-- fp_min
	end
end

local function prevf(x)
	if x>0 then
		return x*(1.0-fp_eps)
	elseif x<0 then
		return x*(1.0+fp_eps)
	else
		return 0	-- -fp_min
	end
end

local function ia_new(amin,amax)
	local a={}
	a.min=amin
	a.max=amax
	return setmetatable(a,ia)
end

local function ia_round(a)
	a.min=prevf(a.min)
	a.max=nextf(a.max)
	return a
end

function ia.range(a)
	return a.min,a.max
end

function ia.__unm(b)
	local a=ia_new()
	a.min=-b.max
	a.max=-b.min
	return a
end

function ia.__add(b,c)
	local a=ia_new()
	if type(b)=="number" then
		a.min=b+c.min
		a.max=b+c.max
	elseif type(c)=="number" then
		a.min=c+b.min
		a.max=c+b.max
	else
		a.min=b.min+c.min
		a.max=b.max+c.max
	end
	return ia_round(a)
end

function ia.__sub(b,c)
	local a=ia_new()
	if type(b)=="number" then
		a.min=b-c.max
		a.max=b-c.min
	elseif type(c)=="number" then
		a.min=b.min-c
		a.max=b.max-c
	else
		a.min=b.min-c.max
		a.max=b.max-c.min
	end
	return ia_round(a)
end

local function ia_scale(b,t)
	local a=ia_new()
	if t>=0 then
		a.min=t*b.min
		a.max=t*b.max
	else
		a.min=t*b.max
		a.max=t*b.min
	end
	return ia_round(a)
end

function ia.__mul(b,c)
	if type(b)=="number" then
		return ia_scale(c,b)
	elseif type(c)=="number" then
		return ia_scale(b,c)
	end
	local amin,amax
	if (b.min>=0.0) then
		if (c.min>=0.0) then
			amin=b.min*c.min
			amax=b.max*c.max
		elseif (c.max<=0.0) then
			amin=b.max*c.min
			amax=b.min*c.max
		else
			amin=b.max*c.min
			amax=b.max*c.max
		end
	elseif (b.max<=0.0) then
		if (c.min>=0.0) then
			amin=b.min*c.max
			amax=b.max*c.min
		elseif (c.max<=0.0) then
			amin=b.max*c.max
			amax=b.min*c.min
		else
			amin=b.min*c.max
			amax=b.min*c.min
		end
	else
		if (c.min>=0.0) then
			amin=b.min*c.max
			amax=b.max*c.max
		elseif (c.max<=0.0) then
			amin=b.max*c.min
			amax=b.min*c.min
		else
			amin=math.min(b.min*c.max,b.max*c.min)
			amax=math.max(b.min*c.min,b.max*c.max)
		end
	end
	return ia_round(ia_new(amin,amax))
end

function ia.__mul(b,c)
	if type(b)=="number" then
		return ia_scale(c,b)
	elseif type(c)=="number" then
		return ia_scale(b,c)
	end
        local a1,a2,a3,a4=b.min*c.min,b.min*c.max,b.max*c.min,b.max*c.max
	local amin=math.min(a1,a2,a3,a4)
	local amax=math.max(a1,a2,a3,a4)
	return ia_round(ia_new(amin,amax))
end

function ia.__pow(b,c)
	assert(c>0 and c==math.floor(c))
	--if c==2 then return b*b end
	--if c==3 then return b*b*b end
	local a=ia_new()
	if (c%2)==1 then
		a.min=b.min^c
		a.max=b.max^c
	else
		if b.min>=0 then
			a.min=b.min^c
			a.max=b.max^c
		elseif b.max<=0 then
			a.min=b.max^c
			a.max=b.min^c
		else
			a.min=0.0
			if b.max<-b.min then
				a.max=b.min^c
			else
				a.max=b.max^c
			end
		end
	end
	return ia_round(a)
end

ia.new=ia_new
ia.round=ia_round
ia.interval=ia_new

return ia
