local N=6
local outer=false

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function circle(k)
	local t=math.rad(k*360/N)
	return math.cos(t),math.sin(t)
end

local function plot(xmin,xmax,ymin,ymax)
	local R=1
	if outer then
		love.graphics.setColor(0,0,255)
		love.graphics.print(string.format("π≥%.8f  n=%d",R*N*math.sin(math.pi/N),N),30,30)
		R=1/math.cos(math.pi/N)
		love.graphics.setColor(255,0,0)
		love.graphics.print(string.format("π≤%.8f",R*N*math.sin(math.pi/N)),30,100)
	else
		love.graphics.setColor(100,100,100)
		love.graphics.print(string.format("π≈%.8f  n=%d",R*N*math.sin(math.pi/N),N),30,30)
	end
	coords(xmin,xmax,ymin,ymax)
	-- outer polygon
	love.graphics.setColor(255,200,200)
	love.graphics.setLineWidth(0)
	--love.graphics.circle("line",0,0,R,60)
	love.graphics.setColor(255,0,0)
	love.graphics.setLineWidth(0.01)
	for k=0,N do
		local x1,y1=circle(k)
		local x2,y2=circle(k+1)
		love.graphics.line(R*x1,R*y1,R*x2,R*y2)
	end
	-- inner polygon
	love.graphics.setColor(200,200,255)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0)
	love.graphics.circle("line",0,0,1,60)
	love.graphics.setColor(0,0,255)
	love.graphics.setLineWidth(0.01)
	for k=0,N do
		local x1,y1=circle(k)
		local x2,y2=circle(k+1)
		love.graphics.line(x1,y1,x2,y2)
	end
	if outer then return end
	-- inner vertices
	love.graphics.setPointSize(15)
	love.graphics.setColor(200,200,255)
	love.graphics.setLineWidth(0)
	for k=0,N do
		local x,y=circle(k)
		love.graphics.points(x,y)
	end
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(1280,1280,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("Archimedes and π")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(48))
	love.keyboard.setKeyRepeat(true)
end

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N/2
		if N<3 then N=3 end
	elseif key == ']' then
		N=2*N
	elseif key == 'o' then
		outer= not outer
	end
end

function love.draw()
	plot(-1.5,1.5,-1.2,1.5)
end
