local ia=require"ia"
local interval=ia.interval
local range=ia.range

local function f(x)
	--return (x+1.2)*(x-1.2)*(x-2)*(x-0.1)
	--return (x+1.2)*(x-1.2)*(x-2)*(x-0.1)*x^2
	--return (x+1.2)*(x-1.2)*(x-2)*x
	--return (x+1.2)*(x-1.2)*(x-2)*x^2
	--return (x-1)^6
	--return x*(x*(x*(x*(x*(x-6)+15)-20)+15)-6)+1
	--return x*(x*(x*(x-2.1)-1.24)+3.024)-0.288
	--return x^3-x-1
	--return x^4-2.1*x^3-1.24*x^2+3.024*x-0.288
	--return x^5-2*x^4-1.44*x^3+2.88*x^2
	--return x^6-6*x^5+15*x^4-20*x^3+15*x^2-6*x+1
	return (x+1.2)*(x-1.2)*(x-2)*(x-0.1)
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function axes(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,200)
	love.graphics.setLineWidth(0)
	love.graphics.line(xmin,0,xmax,0)
end

local function graph(xmin,xmax,ymin,ymax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dx=(xmax-xmin)/n
	local x1,y1=xmin,f(xmin)
	local x2,y2
	for x=xmin,xmax,dx do
		x2,y2=x,f(x)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=xmax,f(xmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function accept(xmin,xmax,ymin,ymax)
	love.graphics.setColor(255,200,200)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(255,0,0)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function discard(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,255)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(0,0,255)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local accepted

local function explore(xmin,xmax,level)
	local x=interval(xmin,xmax)
	local y=f(x)
	local ymin,ymax=range(y)
	if ymax<0 or ymin>0 then
		discard(xmin,xmax,ymin,ymax)
	elseif level==0 then
		accept(xmin,xmax,ymin,ymax)
		accepted=accepted+1
	else
		local xmid=(xmin+xmax)/2
		explore(xmin,xmid,level-1)
		explore(xmid,xmax,level-1)
	end
end

local function result(n,accepted)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%d\t%d",n,accepted),10,10)
end

local function plot(xmin,xmax,ymin,ymax,n)
	accepted=0
	coords(xmin,xmax,ymin,ymax)
	axes(xmin,xmax,ymin,ymax)
	explore(xmin,xmax,n)
	graph(xmin,xmax,ymin,ymax,128)
	result(n,accepted)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("solving equations with interval arithmetic")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	end
end

function love.draw()
	local xmin,xmax=-1.5,2.2
	local ymin,ymax=-0.3,0.3
	local ymin,ymax=-3,3
	plot(xmin,xmax,ymin,ymax,N)
end
