local aa=require"aa1"
local interval=aa.interval
local range=aa.range
local plotbox=true
local plotgraph=true
local plotzonotope=true
local T=0.5

local function f(x)
	--return x^2-1
	--return x*(1-x)
	--return x*(x^2-1)-0.125
	--return x^3-x-0.125
	--return x*x*x-x-0.125
	--return 0.25*x*(x-1)*(x+1)*(x-2)*(x+2)-0.123
	--return 0.25*x^5 - 1.25*x^3 + x - 0.123
	--return 0.25*x*(x^2-1)*(x^2-4)-0.123
	--return x^3*(0.25*x^2 - 1.25) + x - 0.123
	--return x*(x^2-1)-1
	--return x^3-x-1
	--return x^6-6*x^5+15*x^4-20*x^3+15*x^2-6*x+1
	--return (x+1.2)*(x-1.2)*(x-2)*x
	--return x^4*x^2 --(x^3)^2
	return x^3
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function axes(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,200)
	love.graphics.setLineWidth(0)
	love.graphics.line(xmin,0,xmax,0)
	love.graphics.line(0,ymin,0,ymax)
end

local function graph(xmin,xmax,ymin,ymax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dx=(xmax-xmin)/n
	local x1,y1=xmin,f(xmin)
	local x2,y2
	for x=xmin,xmax,dx do
		x2,y2=x,f(x)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=xmax,f(xmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function box(x,y)
	local xmin,xmax=range(x)
	local ymin,ymax=range(y)
	if plotbox then
		love.graphics.setColor(200,200,255)
		love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
		love.graphics.setColor(0,0,255)
		love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
	end
	return (xmax-xmin)*(ymax-ymin)
end

local function zonotope(x,y)
	local x1=x[0]-x[1]
	local y1=y[0]-y[1]-y[2]
	local x2=x[0]-x[1]
	local y2=y[0]-y[1]+y[2]
	local x3=x[0]+x[1]
	local y3=y[0]+y[1]+y[2]
	local x4=x[0]+x[1]
	local y4=y[0]+y[1]-y[2]
	if plotzonotope then
		love.graphics.setColor(255,200,200)
		love.graphics.polygon("fill",x1,y1,x2,y2,x3,y3,x4,y4)
		love.graphics.setColor(255,0,0)
		love.graphics.polygon("line",x1,y1,x2,y2,x3,y3,x4,y4)
	end
	return 2*y[2]*2*x[1]
end

local function visit(xmin,xmax,n)
	local xmid=xmin+T*(xmax-xmin)
	local dx=(xmax-xmin)/n/2
	local x=interval(xmid-dx,xmid+dx)
	local y=f(x)
	local ab=box(x,y)
	local az=zonotope(x,y)
	love.graphics.setColor(100,100,100)
	love.graphics.setColor(100,100,255)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	love.graphics.line(xmid-dx,0,xmid+dx,0)
	return 2*dx,ab,az,az/ab
end

local function result(dx,ab,az,n)
	local ratio=az/ab
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%0.3f",ratio),10,10)
	--love.graphics.print(string.format("%0.3f\t%0.3f\t%0.3f\t%0.3f",dx,ab/dx^2,az/dx^3,ratio/dx),10,10)
end

local function plot(xmin,xmax,ymin,ymax,n)
	coords(xmin,xmax,ymin,ymax)
	axes(xmin,xmax,ymin,ymax)
	local dx,ab,az=visit(xmin,xmax,n)
	if plotgraph then
		graph(xmin,xmax,ymin,ymax,128)
	end
	result(dx,ab,az,n)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("zonotope approximaton with affine arithmetic")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
	love.graphics.setPointSize(8)
end

local N=0

function love.keypressed(key)
	local dt=2^-6
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	elseif key == 'b' then
		plotbox= not plotbox
	elseif key == 'g' then
		plotgraph= not plotgraph
	elseif key == 'z' then
		plotzonotope= not plotzonotope
	elseif key == 'left' then
		T=T-dt
	elseif key == 'right' then
		T=T+dt
	end
end

function love.draw()
	local xmin,xmax=-2,3
	local ymin,ymax=-2,2
	local xmin,xmax=-2,2
	plot(xmin,xmax,ymin,ymax,2^N)
end
